unit GNOMECheckBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, GNOMEAboutInfo;

type
  TGNOMECheckBox = class(TButtonControl)
  private
    FAbout: TGNOMEAboutInfo;
    FAlignment: TLeftRight;
    FAllowGrayed: Boolean;
    FBkColor: TColor;
    FBorderColor: TColor;
    FBuffer: TBitmap;
    FCanvas: TCanvas;
    FMouseInControl: Boolean;
    FState: TCheckBoxState;
    FFocused: Boolean;

    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMMOUSEENTER(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMOUSELEAVE(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
    procedure CNDrawItem(var Message: TWMDrawItem); message CN_DRAWITEM;
    procedure DrawItem(const DrawItemStruct: TDrawItemStruct);
    function GetChecked: Boolean;
    procedure SetAlignment(Value: TLeftRight);
    procedure SetBkColor(Value: TColor);
    procedure SetBorderColor(Value: TColor);
    procedure SetChecked(Value: Boolean);
    procedure SetState(Value: TCheckBoxState);
    procedure WMSize(var Message: TMessage); message WM_SIZE;
    procedure WMKillFocus (var Message: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMSetFocus (var Message: TWMSetFocus); message WM_SETFOCUS;
  protected
    procedure Click; override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
    procedure Toggle; virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetControlsAlignment: TAlignment; override;
  published
    property About: TGNOMEAboutInfo read FAbout write FAbout stored False;
    property Alignment: TLeftRight read FAlignment write SetAlignment default taRightJustify;
    property AllowGrayed: Boolean read FAllowGrayed write FAllowGrayed default False;
    property BackgroundColor: TColor read FBkColor write SetBkColor default $EAEAEA;
    property BorderColor: TColor read FBorderColor write SetBorderColor default clBlack;
    property Action;
    property Anchors;
    property BiDiMode;
    property Caption;
    property Checked: Boolean read GetChecked write SetChecked stored False;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property State: TCheckBoxState read FState write SetState default cbUnchecked;
    property TabOrder;
    property TabStop default True;
    property Visible;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;


    {
    property Constraints;
    property Ctl3D;
    property DragKind;
    }
    property ParentBiDiMode;
    property ParentCtl3D;
    property OnContextPopup;
    property OnEndDock;
    property OnStartDock;

  end;

implementation

function TGNOMECheckBox.GetControlsAlignment: TAlignment;
begin
  if not UseRightToLeftAlignment then
    Result := taRightJustify
  else
    if FAlignment = taRightJustify then
      Result := taLeftJustify
    else
      Result := taRightJustify;
end;

procedure TGNOMECheckBox.SetAlignment(Value: TLeftRight);
begin
  if FAlignment <> Value then
  begin
    FAlignment := Value;
    RecreateWnd;
  end;
end;

{ Cause click to do changed too }
procedure TGNOMECheckBox.Click;
begin
  inherited Click;
end;

{ Component message that enabled state has changed - redraw }
procedure TGNOMECheckBox.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  Repaint;
end;

{ Create instance of this control }
constructor TGNOMECheckBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCanvas := TCanvas.Create;
  FBuffer := TBitmap.Create;
  FBuffer.Width := Width;
  FBuffer.Height := Height;
  FState := cbUnchecked;
  ControlStyle := [csSetCaption, csOpaque];
  Width := 140;
  Height := 24;
  TabStop := True;
  FBkColor := $00EAEAEA;
end;

destructor TGNOMECheckBox.Destroy;
begin
  inherited Destroy;
  FBuffer.Free;
  FCanvas.Free;
end;

{ Return checked status }
function TGNOMECheckBox.GetChecked: Boolean;
begin
  Result := State = cbChecked;
end;

{ Set checked status }
procedure TGNOMECheckBox.SetChecked(Value: Boolean);
begin
  if Value then
     State := cbChecked
  else
     State := cbUnchecked;
  Repaint;
end;

{ Set state }
procedure TGNOMECheckBox.SetState(Value: TCheckBoxState);
begin
  if (FState <> Value) then
     begin
       FState := Value;
       Repaint;
       if HandleAllocated then
          SendMessage(Handle, BM_SETCHECK, Integer(FState), 0);
       Click;
     end;


end;

{ Create owner-draw control }
procedure TGNOMECheckBox.CreateParams(var Params: TCreateParams);
const
  Alignments: array[Boolean, TLeftRight] of DWORD =
    ((BS_LEFTTEXT, 0), (0, BS_LEFTTEXT));
begin
  inherited CreateParams(Params);
  CreateSubClass(Params, 'BUTTON');
  with Params do
    begin
      Style := Style or BS_OWNERDRAW or Alignments[UseRightToLeftAlignment, FAlignment];;
      WindowClass.style := WindowClass.style and not (CS_HREDRAW or CS_VREDRAW);
    end;
end;

{ Create windows control }
procedure TGNOMECheckBox.CreateWnd;
begin
  inherited CreateWnd;
  SendMessage(Handle, BM_SETCHECK, Integer(FState), 0);
end;

{ Redraw on resize - control may be able to display more }
procedure TGNOMECheckBox.WMSize(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

///////////////////////////////////////////////////////////////////////////

{ Routine to toggle between states }
procedure TGNOMECheckBox.Toggle;
begin
  case State of
    cbUnchecked:
      if AllowGrayed then
         State := cbGrayed
      else
         State := cbChecked;
    cbChecked: State := cbUnchecked;
    cbGrayed: State := cbChecked;
  end;
  Repaint;
end;






{ Handle window accelerated keys }
procedure TGNOMECheckBox.CMDialogChar(var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(CharCode, Caption) and CanFocus then
    begin
      SetFocus;
      if Focused then Toggle;
      Result := 1;
    end
  else
    inherited;
end;

{ Component notification of click }
procedure TGNOMECheckBox.CNCommand(var Message: TWMCommand);
begin
  if (Message.NotifyCode = BN_CLICKED) then Toggle;
end;

{ Component notification to draw }
procedure TGNOMECheckBox.CNDrawItem(var Message: TWMDrawItem);
begin
  DrawItem(Message.DrawItemStruct^);
end;



{ Actually draw the control }
procedure TGNOMECheckBox.DrawItem(const DrawItemStruct: TDrawItemStruct);
var
  IsDown, IsDefault: Boolean;
  Rec, BtnR: TRect;
  BtnTop: Integer;
  FilCol, BorCol, CapCol, T1, B1, S1, S2, BkColor: TColor;
begin
  { Assign image and canvas }
  FBuffer.Width := Self.Width;
  FBuffer.Height := Self.Height;
  FCanvas.Handle := DrawItemStruct.hDC;
  Rec := ClientRect;

  { Decode the state of the button }
  with DrawItemStruct do
  begin
    IsDown := itemState and ODS_SELECTED <> 0;
    IsDefault := itemState and ODS_FOCUS <> 0;
  end;

  with FBuffer.Canvas do
  begin
    if Enabled then
    begin
      if FMouseInControl then BkColor := FBkColor else BkColor := Self.Color;
      Font := Self.Font;
      if FFocused then Pen.Color := FBorderColor else Pen.Color := FBkcolor;
      Pen.Style := psSolid;
      Brush.Style := bsSolid;
      Brush.Color := BkColor;
      Rectangle(ClientRect);




      If State=cbUnchecked then
      begin
        Pen.Color := clWhite;
        BtnTop := (ClientRect.Bottom - ClientRect.Top - 10) div 2 + 1;
        MoveTo(3, BtnTop);
        LineTo(12, BtnTop);
        MoveTo(3, BtnTop);
        LineTo(3, BtnTop + 9);

        Pen.Color := clBlack;
        MoveTo(12, BtnTop);
        LineTo(12, BtnTop + 10);
        MoveTo(3, BtnTop + 9);
        LineTo(13, BtnTop + 9);

        Brush.Color := $00EAEAEA;
        BtnR.Top := BtnTop + 1;
        BtnR.Bottom := BtnTop + 9;
        BtnR.Left := 4;
        BtnR.Right := 12;
        FillRect(BtnR);

        Pen.Color := $00A4A4A4;
        MoveTo(11, BtnTop + 2);
        LineTo(11, BtnTop + 9);
        MoveTo(5, BtnTop + 8);
        LineTo(12, BtnTop + 8);

      end;

    If State=cbChecked then
    begin
      Pen.Color := $00888888;
        BtnTop := (ClientRect.Bottom - ClientRect.Top - 10) div 2 + 1;
        MoveTo(3, BtnTop);
        LineTo(13, BtnTop);
        MoveTo(3, BtnTop);
        LineTo(3, BtnTop + 10);

        Pen.Color := clWhite;
        MoveTo(12, BtnTop + 1);
        LineTo(12, BtnTop + 10);
        MoveTo(4, BtnTop + 9);
        LineTo(13, BtnTop + 9);

        Brush.Color := $00C3C3C3;
        BtnR.Top := BtnTop + 1;
        BtnR.Bottom := BtnTop + 9;
        BtnR.Left := 4;
        BtnR.Right := 12;
        FillRect(BtnR);

        Pen.Color := clBlack;
        MoveTo(4, BtnTop + 1);
        LineTo(4, BtnTop + 8);
        MoveTo(4, BtnTop + 1);
        LineTo(11, BtnTop + 1);
    end;

    end;


    Brush.Color := Self.Color;
    Brush.Style := bsSolid;
    Rec.Left := Rec.Left + 17;
    SetBkMode(Handle, TRANSPARENT);
    DrawText(Handle, PChar(Caption), Length(Caption), Rec, DT_VCENTER or DT_SINGLELINE);
    
  end;

  { Paint completed image & release }
  FCanvas.Draw(0, 0, FBuffer);
  FCanvas.Handle := 0;
end;

procedure TGNOMECheckBox.SetBkColor(Value: TColor);
begin
  if (Value <> FBkColor) then
  begin
    FBkColor := Value;
    Repaint;
  end;
end;

procedure TGNOMECheckBox.SetBorderColor(Value: TColor);
begin
  if (Value <> FBkColor) then
  begin
    FBorderColor := Value;
    Repaint;
  end;
end;



procedure TGNOMECheckBox.WMKillFocus (var Message: TWMKillFocus);
begin
  inherited;
  if Enabled then
  begin
    //FMouseInControl := False;
    FFocused := False;
    Repaint;
  end;
end;

procedure TGNOMECheckBox.WMSetFocus (var Message: TWMSetFocus);
begin
  inherited;
  if Enabled then
  begin
    FFocused := True;
    //FMouseInControl := False;
    Repaint;
  end;
end;


procedure TGNOMECheckBox.CMMOUSEENTER(var Message: TMessage);
begin
  if Enabled then
  begin
    FMouseInControl := True;
    Repaint;
  end;
end;

procedure TGNOMECheckBox.CMMOUSELEAVE(var Message: TMessage);
begin
  if Enabled then
  begin
    FMouseInControl := False;
    Repaint;
  end;
end;

end.
