unit GNOMEGroupBox;

{
  TGNOMEGroupBox Is Part Of The Suite GNOME Style

  Feb, 10, 2001  Start Working On This Component
  Feb, 10, 2001  Finish The Component

}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  GNOMEAboutInfo;

type
  TGNOMEGroupBox = class(TCustomControl)
  private
    FAbout: TGNOMEAboutInfo;
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure WMSize(var Message: TMessage); message WM_SIZE;
  protected
    procedure AdjustClientRect(var Rect: TRect); override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property About: TGNOMEAboutInfo read FAbout write FAbout stored False;
    property Align;
    property Anchors;
    property BiDiMode;
    property Caption;
    property Color;
    property Constraints;
    property Ctl3D;
    property DockSite;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDockDrop;
    property OnDockOver;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnGetSiteInfo;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
    property OnUnDock;
  end;


implementation

constructor TGNOMEGroupBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csAcceptsControls, csCaptureMouse, csClickEvents,
    csSetCaption, csDoubleClicks, csReplicatable];
  Width := 185;
  Height := 105;
end;

procedure TGNOMEGroupBox.AdjustClientRect(var Rect: TRect);
begin
  inherited AdjustClientRect(Rect);
  Canvas.Font := Font;
  Inc(Rect.Top, Canvas.TextHeight('0'));
  InflateRect(Rect, -1, -1);
  if Ctl3d then InflateRect(Rect, -1, -1);
end;

procedure TGNOMEGroupBox.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params.WindowClass do
    style := style and not (CS_HREDRAW or CS_VREDRAW);
end;

procedure TGNOMEGroupBox.Paint;
var
  H, UpperLine: Integer;
  R: TRect;
  Flags: Longint;
begin
  with Canvas do
  begin
    Font := Self.Font;
    H := TextHeight('0');
    R := Rect(0, H div 2 - 1, Width, Height);
    if Ctl3D then
    begin
      Inc(R.Left);
      Inc(R.Top);
      Brush.Color := clWhite;
      FrameRect(R);
      OffsetRect(R, -1, -1);
      Brush.Color := $00969696;
      FrameRect(R);
      Inc(R.Bottom);
      Inc(R.Right);
      Pixels[R.Left, R.Bottom - 1] := clWhite;
      Pixels[R.Right - 1, R.Top] := clWhite;
      Pixels[R.Right - 2, R.Top + 1] := clWhite;
    end else
      begin
        Brush.Color := clBlack;
        FrameRect(R);
      end;
    if Text <> '' then
    begin
      UpperLine := R.Top;
      if not UseRightToLeftAlignment then
        R := Rect(5, 0, 0, H)
      else
        R := Rect(R.Right - Canvas.TextWidth(Text) - 5, 0, 0, H);
      Flags := DrawTextBiDiModeFlags(DT_SINGLELINE);
      DrawText(Handle, PChar(Text), Length(Text), R, Flags or DT_CALCRECT);
      Brush.Color := Color;
      DrawText(Handle, PChar(Text), Length(Text), R, Flags);
      if Ctl3D then
      begin
        Pixels[4, UpperLine] := clWhite;
        Pixels[4, UpperLine + 1] := Color;
        Pixels[5, UpperLine] := Color;
        Pixels[5, UpperLine + 1] := Color;
        Pixels[R.Right, UpperLine] := clWhite;
        Pixels[R.Right, UpperLine + 1] := Color;
      end;
    end;
  end;
end;

procedure TGNOMEGroupBox.CMDialogChar(var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(CharCode, Caption) and CanFocus then
    begin
      SelectFirst;
      Result := 1;
    end else
      inherited;
end;

procedure TGNOMEGroupBox.CMTextChanged(var Message: TMessage);
begin
  Invalidate;
  Realign;
end;

procedure TGNOMEGroupBox.CMCtl3DChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
  Realign;
end;

procedure TGNOMEGroupBox.WMSize(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;


end.
 