unit GNOMEProgressBar;

{
  TGNOMEProgressBar Is Part Of The Suite GNOME Style

  Feb, 10, 2001  Start Working On This Component
  Feb, 10, 2001  Finish The Component

  
}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Comctrls, Consts, GNOMEAboutInfo;

const
  Limit16 = 65535;

type
  TGNOMEProgressBar = class(TGraphicControl)
  private
    FAbout: TGNOMEAboutInfo;
    FShadowColor: TColor;
    FInnerShadowColor: TColor;
    FBackgroundColor: TColor;
    FHighlightColor: TColor;
    FBarColor: TColor;
    FBarShadowColor: TColor;
    FBarHighlightColor: TColor;
    FPosition: Integer;
    FMin, FMax : Integer;
    FOnEnter: TNotifyEvent;
    FOnExit: TNotifyEvent;
    FStep: Integer;
    FOrientation: TProgressBarOrientation;
    procedure SetColors(Index: Integer; Value: TColor);
    procedure SetPosition(Value: Integer);
    procedure SetStep(Value: Integer);
    procedure SetMax(Value: Integer);
    procedure SetMin(Value: Integer);
    //procedure SetOrientation(Value: TProgressBarOrientation);
    procedure CMEnter(var Message: TCMEnter); message CM_ENTER;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
  protected
    procedure Paint; override;

    destructor Destroy;
    procedure DoEnter; dynamic;
    procedure DoExit; dynamic;
  public
    constructor Create(AOwner: TComponent); override;
    procedure StepIt;
    procedure StepBy(Delta: Integer);
  published
    property About: TGNOMEAboutInfo read FAbout write FAbout stored False;
    property Align;
    property Anchors;
    property Enabled;
    property Hint;
    property BackgroundColor: TColor index 0 read FBackgroundColor write SetColors default clWhite;
    property ShadowColor: TColor  index 1 read FShadowColor write SetColors default $00B3B3B3;
    property InnerShadowColor: TColor index 2 read FInnerShadowColor write SetColors default clBlack;
    property HighlightColor: TColor index 3 read FHighlightColor write SetColors default clWhite;
    property BarColor: TColor index 4 read FBarColor write SetColors default $00A0A0A0;
    property BarShadowColor: TColor  index 5 read FBarShadowColor write SetColors default $00707070;
    property BarHighlightColor: TColor  index 6 read FBarHighlightColor write SetColors default $00D0D0D0;
    property Position: Integer read FPosition write SetPosition default 50;
    property Max: integer read FMax write SetMax;
    property Min: integer read FMin write SetMin;
    property Step: Integer read FStep write SetStep default 10;

    property Visible;
    property OnContextPopup;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter: TNotifyEvent read FOnEnter write FOnEnter;
    property OnExit: TNotifyEvent read FOnExit write FOnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

//  procedure Register;

implementation

constructor TGNOMEProgressBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FBackgroundColor := clWhite;
  FShadowColor := $00B3B3B3;
  FInnerShadowColor := clBlack;
  FBarColor := $00A0A0A0; 
  FHighlightColor := clWhite;
  FBarShadowColor := $00707070;
  FBarHighlightColor := $00D0D0D0;

  FPosition := 50;
  Height := GetSystemMetrics(SM_CYVSCROLL);
  Width := 150;
  ControlStyle := ControlStyle + [csOpaque];
  FStep := 10;
  FMax:= 100;
  FMin:= 0;
  FOrientation := pbHorizontal;
end;

destructor TGNOMEProgressBar.Destroy;
begin
  inherited;
end;

procedure TGNOMEProgressBar.SetColors(Index: Integer; Value: TColor);
begin
  case Index of
    0: FBackgroundColor := Value;
    1: FShadowColor := Value;
    2: FInnerShadowColor := Value;
    3: FHighlightColor := Value;
    4: FBarColor := Value;
    5: FBarShadowColor := Value;
    6: FBarHighlightColor := Value;
  end;
  Repaint;
end;

procedure TGNOMEProgressBar.Paint;
var
  DrawPos: Integer;
  R: TRect;
  TotalWidth: Integer;
begin
  with Canvas do
  begin
    if FOrientation = pbHorizontal then
    begin
      Pen.Color := FShadowColor;
      MoveTo(ClientRect.Left, ClientRect.Top);
      LineTo(ClientRect.Right, ClientRect.Top);
      MoveTo(ClientRect.Left, ClientRect.Top);
      LineTo(ClientRect.Left, ClientRect.Bottom);
      Pen.Color := FHighlightColor;
      MoveTo(ClientRect.Left + 1, ClientRect.Bottom - 1);
      LineTo(ClientRect.Right, ClientRect.Bottom - 1);
      MoveTo(ClientRect.Right - 1, ClientRect.Top + 1);
      LineTo(ClientRect.Right - 1, ClientRect.Bottom);
      Pen.Color := FInnerShadowColor;
      MoveTo(ClientRect.Left + 1, ClientRect.Top + 1);
      LineTo(ClientRect.Right - 1, ClientRect.Top + 1);
      MoveTo(ClientRect.Left + 1, ClientRect.Top + 1);
      LineTo(ClientRect.Left + 1, ClientRect.Bottom - 1);
      R := ClientRect;
      Inc(R.Left);
      Inc(R.Top);
      InflateRect(R, -1, -1);
      Brush.Color := FBackgroundColor;
      FillRect(R);
      TotalWidth := R.Right - R.Left;
      DrawPos := Trunc((FPosition - FMin) / (FMax - FMin) * TotalWidth);
      R.Right := R.Left + DrawPos;
      Dec(R.Bottom);
      Brush.Color := FBarColor;
      FillRect(R);
      Pen.Color := FBarHighlightColor;
      MoveTo(R.Left, R.Top);
      LineTo(R.Right - 1, R.Top);
      MoveTo(R.Left, R.Top);
      LineTo(R.Left, R.Bottom - 1);
      Pen.Color := FInnerShadowColor;
      MoveTo(R.Left, R.Bottom - 1);
      LineTo(R.Right, R.Bottom - 1);
      MoveTo(R.Right - 1, R.Top);
      LineTo(R.Right - 1, R.Bottom);
      Pen.Color := FBarShadowColor;
      MoveTo(R.Left + 2, R.Bottom - 2);
      LineTo(R.Right - 1, R.Bottom - 2);
      MoveTo(R.Right - 2, R.Top + 2);
      LineTo(R.Right - 2, R.Bottom - 1);
    end
    else if FOrientation = pbVertical then
    begin
      Pen.Color := FShadowColor;
      MoveTo(ClientRect.Left, ClientRect.Top);
      LineTo(ClientRect.Right, ClientRect.Top);
      MoveTo(ClientRect.Left, ClientRect.Top);
      LineTo(ClientRect.Left, ClientRect.Bottom);
      Pen.Color := FHighlightColor;
      MoveTo(ClientRect.Left + 1, ClientRect.Bottom - 1);
      LineTo(ClientRect.Right, ClientRect.Bottom - 1);
      MoveTo(ClientRect.Right - 1, ClientRect.Top + 1);
      LineTo(ClientRect.Right - 1, ClientRect.Bottom);
      Pen.Color := FInnerShadowColor;
      MoveTo(ClientRect.Left + 1, ClientRect.Top + 1);
      LineTo(ClientRect.Right - 1, ClientRect.Top + 1);
      MoveTo(ClientRect.Left + 1, ClientRect.Top + 1);
      LineTo(ClientRect.Left + 1, ClientRect.Bottom - 1);
      R := ClientRect;
      Inc(R.Left);
      Inc(R.Top);
      InflateRect(R, -1, -1);
      Brush.Color := FBackgroundColor;
      FillRect(R);
      TotalWidth := R.Right - R.Left;
      DrawPos := Trunc((FPosition - FMin) / (FMax - FMin) * TotalWidth);
      R.Right := R.Left + DrawPos;
      Dec(R.Bottom);
      Brush.Color := FBarColor;
      FillRect(R);
      Pen.Color := FBarHighlightColor;
      MoveTo(R.Left, R.Top);
      LineTo(R.Right - 1, R.Top);
      MoveTo(R.Left, R.Top);
      LineTo(R.Left, R.Bottom - 1);
      Pen.Color := FInnerShadowColor;
      MoveTo(R.Left, R.Bottom - 1);
      LineTo(R.Right, R.Bottom - 1);
      MoveTo(R.Right - 1, R.Top);
      LineTo(R.Right - 1, R.Bottom);
      Pen.Color := FBarShadowColor;
      MoveTo(R.Left + 2, R.Bottom - 2);
      LineTo(R.Right - 1, R.Bottom - 2);
      MoveTo(R.Right - 2, R.Top + 2);
      LineTo(R.Right - 2, R.Bottom - 1);
    end;
  end;
end;

procedure TGNOMEProgressBar.SetMax(Value: Integer);
begin
  if (Value <> FMax) and (Value > FMin) and (Value >= FPosition) then
  begin
    FMax := Value;
    Repaint;
  end;
end;

procedure TGNOMEProgressBar.SetMin(Value: Integer);
begin
  if (Value <> FMin) and (Value < FMax) and (Value <= FPosition) then
  begin
    FMin := Value;
    Repaint;
  end;
end;

procedure TGNOMEProgressBar.SetPosition(Value: Integer);
begin
  if (Value <> FPosition) and (Value >= FMin) and (Value <= FMax) then
  begin
    FPosition := Value;
    Repaint;
  end;
end;

procedure TGNOMEProgressBar.DoEnter;
begin
  if Assigned(FOnEnter) then FOnEnter(Self);
end;

procedure TGNOMEProgressBar.DoExit;
begin
  if Assigned(FOnExit) then FOnExit(Self);
end;

procedure TGNOMEProgressBar.CMEnter(var Message: TCMEnter);
begin
  if SysLocale.MiddleEast then
    if UseRightToLeftReading then
    begin
      if Application.BiDiKeyboard <> '' then
        LoadKeyboardLayout(PChar(Application.BiDiKeyboard), KLF_ACTIVATE);
    end
    else
      if Application.NonBiDiKeyboard <> '' then
        LoadKeyboardLayout(PChar(Application.NonBiDiKeyboard), KLF_ACTIVATE);
  DoEnter;
end;

procedure TGNOMEProgressBar.CMExit(var Message: TCMExit);
begin
  DoExit;
end;

procedure TGNOMEProgressBar.StepIt;
begin
  SetPosition(FPosition + FStep);
  Repaint;
end;

procedure TGNOMEProgressBar.StepBy(Delta: Integer);
begin
  SetPosition(FPosition + Delta);
  Repaint;
end;


procedure TGNOMEProgressBar.SetStep(Value: Integer);
begin
  if (Value <> FStep) and (Value >= 0) and (Value <= FMax) then
    FStep := Value;
end;

procedure ProgressLimitError;
begin
  raise Exception.CreateResFmt(@SOutOfRange, [0, Limit16]);
end;


end.
